import os
from typing import Optional
from urllib.parse import urlparse, urlunparse, urljoin

from requests import Session

from oli.tracing.tracing import TracingCollector


def _clean_url(url: str) -> str:
    if not url.startswith("http://") and not url.startswith("https://"):
        raise ValueError("URL must start with 'http://' or 'https://'")
    # remove any path in the url
    parsed_url = urlparse(url)
    cleaned_url = (
        urlunparse((parsed_url.scheme, parsed_url.netloc, "", "", "", "")) + "/"
    )
    return cleaned_url


def ensure_value(value: Optional[str], env_var: str) -> str:
    if value:
        return value
    if not (env_value := os.getenv(env_var)):
        raise ValueError(
            f"Environment variable {env_var} is not set and no value provided."
        )
    return env_value


class OliTracing:
    def __init__(
        self,
        endpoint_url: Optional[str] = None,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
    ):
        endpoint_url = ensure_value(endpoint_url, "OLI_TRACING_ENDPOINT_URL")
        client_id = ensure_value(client_id, "OLI_TRACING_CLIENT_ID")
        client_secret = ensure_value(client_secret, "OLI_TRACING_CLIENT_SECRET")
        self.endpoint_url = _clean_url(endpoint_url)
        self._client_id = client_id
        self._client_secret = client_secret
        self._setup_client()
        self._session = Session()
        self._session.headers.update(self.auth_headers)

    @property
    def auth_headers(self):
        return {
            "CF-Access-Client-Id": self._client_id,
            "CF-Access-Client-Secret": self._client_secret,
        }

    @property
    def tracing_endpoint_url(self):
        return urljoin(self.endpoint_url, "/v1/traces")

    def is_available(self):
        health_url = urljoin(self.endpoint_url, "/arize_phoenix_version")
        response = self._session.get(health_url)
        if response.status_code == 200:
            print(f"Connected to Active Tracing Server; Version: {response.text}")
            return True
        return False

    def _setup_client(self):
        os.environ["PHOENIX_COLLECTOR_ENDPOINT"] = self.endpoint_url
        os.environ["PHOENIX_CLIENT_HEADERS"] = ",".join(
            f"{k}={v}" for k, v in self.auth_headers.items()
        )

    def setup_collector(
        self,
        project_name: Optional[str] = None,
        model_id: Optional[str] = None,
        model_version: Optional[str] = None,
        log_to_console: bool = False,
        use_batch_processor: bool = True,
    ) -> TracingCollector:
        project_name = ensure_value(project_name, "OLI_TRACING_PROJECT_NAME")
        collector = TracingCollector(
            tracing_client=self,
            project_name=project_name,
            model_id=model_id,
            model_version=model_version,
            log_to_console=log_to_console,
            use_batch_processor=use_batch_processor,
        )
        collector.setup()
        return collector



import os
import typing
from typing import Optional

from openinference.semconv.resource import ResourceAttributes
from opentelemetry import trace
from opentelemetry.exporter.otlp.proto.http.trace_exporter import (
    OTLPSpanExporter as HttpSpanExporter,
)
from opentelemetry.sdk.trace import Resource, TracerProvider
from opentelemetry.sdk.trace.export import (
    BatchSpanProcessor,
    SimpleSpanProcessor,
    ConsoleSpanExporter,
)

if typing.TYPE_CHECKING:
    # avoid circular import
    from oli.tracing import OliTracing


def _create_resource(
    model_id: str,
    model_version: str,
    project_name: str,
) -> Resource:
    attributes = {}
    if model_id:
        attributes["model_id"] = model_id
    if model_version:
        attributes["model_version"] = model_version
    if project_name:
        attributes[ResourceAttributes.PROJECT_NAME] = project_name
    return Resource(attributes=attributes)


def _ensure_proper_url(url: str) -> str:
    # parse url and ensure the path is /v1/traces
    if not url.startswith("http://") and not url.startswith("https://"):
        raise ValueError("URL must start with 'http://' or 'https://'")
    if not url.endswith("/v1/traces") and not url.endswith("/v1/traces/"):
        return url.rstrip("/") + "/v1/traces"
    return url


class TracingCollector:
    def __init__(
        self,
        tracing_client: "OliTracing",
        project_name: str,
        model_id: Optional[str] = None,
        model_version: Optional[str] = None,
        log_to_console: bool = False,
        use_batch_processor: bool = True,
        use_global_tracer_provider: bool = True,
    ):
        if os.environ.get("OLI_TRACING_USE_GLOBAL_PROVIDER") is None:
            self.use_global_tracer_provider = use_global_tracer_provider
        else:
            self.use_global_tracer_provider = (
                os.environ.get("OLI_TRACING_USE_GLOBAL_PROVIDER").lower() == "true"
            )
        self.tracing_client: "OliTracing" = tracing_client
        self.project_name = project_name
        self.model_id = model_id
        self.model_version = model_version
        self.log_to_console = log_to_console
        self.use_batch_processor = use_batch_processor
        self._setup = False
        self._trace_provider = None

    def setup(self):
        if not isinstance(self.use_batch_processor, bool):
            raise TypeError("use_batch_processor must be of type bool")
        provider = TracerProvider(
            resource=_create_resource(
                self.model_id,
                self.model_version,
                self.project_name,
            )
        )
        processor = (
            BatchSpanProcessor if self.use_batch_processor else SimpleSpanProcessor
        )
        exporter = HttpSpanExporter
        ep = _ensure_proper_url(self.tracing_client.tracing_endpoint_url)
        provider.add_span_processor(
            span_processor=processor(
                span_exporter=exporter(
                    endpoint=ep,
                    headers=self.tracing_client.auth_headers,
                ),
            )
        )
        if self.log_to_console:
            provider.add_span_processor(
                span_processor=processor(
                    span_exporter=ConsoleSpanExporter(),
                )
            )
        self._trace_provider = provider
        if self.use_global_tracer_provider:
            trace.set_tracer_provider(tracer_provider=provider)
        self._setup = True

    @property
    def trace_provider(self):
        return self._trace_provider

    def enable_openai_tracing(self):
        if not self._setup:
            raise RuntimeError("Tracing must be setup before calling this function")
        from openinference.instrumentation.openai import OpenAIInstrumentor

        OpenAIInstrumentor().instrument(trace_provider=self._trace_provider)
        return self

    def enable_dspy_tracing(self):
        if not self._setup:
            raise RuntimeError("Tracing must be setup before calling this function")
        from openinference.instrumentation.dspy import DSPyInstrumentor

        DSPyInstrumentor().instrument(trace_provider=self._trace_provider)
        return self

    def enable_langchain_tracing(self):
        if not self._setup:
            raise RuntimeError("Tracing must be setup before calling this function")
        from openinference.instrumentation.langchain import LangChainInstrumentor

        LangChainInstrumentor().instrument(trace_provider=self._trace_provider)
        return self
